Package: tall
Title: Text Analysis for All
Version: 1.0.0
Authors@R: c(
    person(given = "Massimo", 
           family = "Aria",
           email = "aria@unina.it", 
           role = c("aut", "cre", "cph"),
           comment= c(ORCID = "0000-0002-8517-9411")),
    person(given = "Maria", 
           family = "Spano", 
           email = "maria.spano@unina.it", 
           role = c("aut"),
           comment= c(ORCID = "0000-0002-3103-2342")),
    person(given ="Luca", 
           family = "D'Aniello", 
           email = "luca.daniello@unina.it", 
           role = c("aut"),
           comment= c(ORCID = "0000-0003-1019-9212")),
    person(given = "Corrado", 
           family = "Cuccurullo", 
           email = "corrado.cuccurullo@unicampania.it", 
           role = c("ctb"),
           comment= c(ORCID = "0000-0002-7401-8575")),
    person(given = "Michelangelo", 
           family = "Misuraca", 
           email = "mmisuraca@unisa.it", 
           role = c("ctb"),
           comment= c(ORCID = "0000-0002-8794-966X"))
    )
Description: An R 'shiny' app designed for diverse text analysis tasks, offering a wide range of methodologies tailored to Natural Language Processing (NLP) needs. 
             It is a versatile, general-purpose tool for analyzing textual data. 
             'tall' features a comprehensive workflow, including data cleaning, preprocessing, statistical analysis, and visualization, all integrated for effective text analysis.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://github.com/massimoaria/tall,
        https://massimoaria.github.io/tall-app/
BugReports: https://github.com/massimoaria/tall/issues
Depends: R (>= 3.5.0), shiny
Imports: base64enc, ca, chromote, curl (>= 6.3.0), doParallel, dplyr
        (>= 1.1.0), DT, fontawesome, future, ggplot2, ggraph,
        ggwordcloud, httr2, igraph, jsonlite, later, Matrix, openxlsx,
        pagedown, pdftools (>= 3.6.0), plotly, promises, purrr, ranger,
        Rcpp (>= 1.0.3), readr, readtext, readxl, rlang, RSpectra,
        shinycssloaders (>= 1.1.0), shinydashboardPlus, shinyFiles,
        shinyjs, shinyWidgets, sparkline, stringr, strucchange,
        textrank, tidygraph, tidyr, stm, topicmodels, udpipe, umap,
        visNetwork, word2vec
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
LazyData: true
LinkingTo: Rcpp
NeedsCompilation: yes
Packaged: 2026-04-15 13:42:15 UTC; massimoaria
Author: Massimo Aria [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-8517-9411>),
  Maria Spano [aut] (ORCID: <https://orcid.org/0000-0002-3103-2342>),
  Luca D'Aniello [aut] (ORCID: <https://orcid.org/0000-0003-1019-9212>),
  Corrado Cuccurullo [ctb] (ORCID:
    <https://orcid.org/0000-0002-7401-8575>),
  Michelangelo Misuraca [ctb] (ORCID:
    <https://orcid.org/0000-0002-8794-966X>)
Maintainer: Massimo Aria <aria@unina.it>
Repository: CRAN
Date/Publication: 2026-04-15 14:20:02 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-04-17 08:24:23 UTC; unix
Archs: tall.so.dSYM
