# Generated by roxygen2: do not edit by hand

export(calculate_ngram_is)
export(compute_syntactic_complexity)
export(extract_noun_phrases)
export(extract_svo_triplets)
export(process_multiwords_fast)
export(reinPlot)
export(reinSummary)
export(reinert)
export(tall)
export(term_per_cluster)
export(txt_recode_fast)
export(txt_recode_ngram_fast)
import(Rcpp)
import(curl)
import(doParallel)
import(dplyr)
import(fontawesome)
import(ggraph)
import(ggwordcloud)
import(openxlsx)
import(pagedown)
import(pdftools)
import(purrr)
import(readtext)
import(readxl)
import(shiny)
import(shinyFiles)
import(shinycssloaders)
import(shinydashboardPlus)
import(stm)
import(stringr)
import(tidygraph)
import(tidyr)
import(topicmodels)
import(udpipe)
import(visNetwork)
importFrom(DT,DTOutput)
importFrom(DT,datatable)
importFrom(DT,formatRound)
importFrom(DT,formatStyle)
importFrom(DT,renderDT)
importFrom(Matrix,Matrix)
importFrom(RSpectra,svds)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(base64enc,dataURI)
importFrom(ca,ca)
importFrom(chromote,default_chromote_object)
importFrom(future,plan)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(httr2,req_perform)
importFrom(httr2,request)
importFrom(httr2,resp_body_json)
importFrom(igraph,centr_betw)
importFrom(igraph,cluster_walktrap)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,membership)
importFrom(jsonlite,fromJSON)
importFrom(later,later)
importFrom(plotly,add_annotations)
importFrom(plotly,add_markers)
importFrom(plotly,add_polygons)
importFrom(plotly,add_trace)
importFrom(plotly,config)
importFrom(plotly,event_data)
importFrom(plotly,event_register)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(promises,promise_all)
importFrom(ranger,ranger)
importFrom(readr,write_csv)
importFrom(rlang,sym)
importFrom(shinyWidgets,alert)
importFrom(shinyWidgets,prettyCheckboxGroup)
importFrom(shinyWidgets,updatePrettyCheckboxGroup)
importFrom(shinyjs,useShinyjs)
importFrom(sparkline,sparkline)
importFrom(sparkline,spk_chr)
importFrom(strucchange,breakpoints)
importFrom(textrank,textrank_sentences)
importFrom(umap,umap)
importFrom(word2vec,word2vec)
useDynLib(tall)
