langrepo <- function() {
  language_name <- c(
    "afrikaans",
    "ancient_greek",
    "ancient_greek",
    "arabic",
    "armenian",
    "basque",
    "belarusian",
    "bulgarian",
    "catalan",
    "chinese",
    "chinese",
    "classical_chinese",
    "coptic",
    "croatian",
    "czech",
    "czech",
    "czech",
    "czech",
    "danish",
    "dutch",
    "dutch",
    "english",
    "english",
    "english",
    "english",
    "estonian",
    "estonian",
    "finnish",
    "finnish",
    "french",
    "french",
    "french",
    "galician",
    "galician",
    "german",
    "german",
    "gothic",
    "greek",
    "hebrew",
    "hindi",
    "hungarian",
    "indonesian",
    "irish",
    "italian",
    "italian",
    "italian",
    "italian",
    "italian",
    "japanese",
    "korean",
    "korean",
    "latin",
    "latin",
    "latin",
    "latvian",
    "lithuanian",
    "lithuanian",
    "maltese",
    "marathi",
    "norwegian",
    "norwegian",
    "old_church_slavonic",
    "persian",
    "polish",
    "polish",
    "portuguese",
    "portuguese",
    "romanian",
    "romanian",
    "russian",
    "russian",
    "russian",
    "scottish_gaelic",
    "serbian",
    "slovak",
    "slovenian",
    "slovenian",
    "spanish",
    "spanish",
    "swedish",
    "swedish",
    "tamil",
    "telugu",
    "turkish",
    "ukrainian",
    "urdu",
    "uyghur",
    "vietnamese",
    "wolof"
  )

  treebank <- c(
    "AfriBooms",
    "Perseus",
    "PROIEL",
    "PADT",
    "ArmTDP",
    "BDT",
    "HSE",
    "BTB",
    "AnCora",
    "GSD",
    "GSDSimp",
    "Kyoto",
    "Scriptorium",
    "SET",
    "CAC",
    "CLTT",
    "FicTree",
    "PDT",
    "DDT",
    "Alpino",
    "LassySmall",
    "EWT",
    "GUM",
    "LinES",
    "ParTUT",
    "EDT",
    "EWT",
    "FTB",
    "TDT",
    "GSD",
    "ParTUT",
    "Sequoia",
    "CTG",
    "TreeGal",
    "GSD",
    "HDT",
    "PROIEL",
    "GDT",
    "HTB",
    "HDTB",
    "Szeged",
    "GSD",
    "IDT",
    "ISDT",
    "ParTUT",
    "PoSTWITA",
    "TWITTIRO",
    "VIT",
    "GSD",
    "GSD",
    "Kaist",
    "ITTB",
    "Perseus",
    "PROIEL",
    "LVTB",
    "ALKSNIS",
    "HSE",
    "MUDT",
    "UFAL",
    "Bokmaal",
    "Nynorsk",
    "PROIEL",
    "Seraji",
    "LFG",
    "PDB",
    "Bosque",
    "GSD",
    "Nonstandard",
    "RRT",
    "GSD",
    "SynTagRus",
    "Taiga",
    "ARCOSG",
    "SET",
    "SNK",
    "SSJ",
    "SST",
    "AnCora",
    "GSD",
    "LinES",
    "Talbanken",
    "TTB",
    "MTG",
    "IMST",
    "IU",
    "UDTB",
    "UDT",
    "VTB",
    "WTB"
  )

  description <- c(
    "UD Afrikaans-AfriBooms is a conversion of the AfriBooms Dependency Treebank, originally annotated with a simplified PoS set and dependency relations according to a subset of the Stanford tag set. The corpus consists of public government documents.",
    "This Universal Dependencies Ancient Greek Treebank consists of an automatic conversion of a selection of passages from the Ancient Greek and Latin Dependency Treebank 2.1.",
    "UD_Ancient_Greek-PROIEL is converted from the Ancient Greek data in the PROIEL treebank, and consists of the New Testament plus selections from Herodotus.",
    "The Arabic-PADT UD treebank is based on the",
    "A Universal Dependencies treebank for Eastern Armenian developed for UD originally by the ArmTDP team led by Marat M. Yavrumyan at the Yerevan State University.",
    "The Basque UD treebank is based on a automatic conversion from part of the Basque Dependency Treebank (BDT), created at the University of of the Basque Country by the IXA NLP research group. The treebank consists of 8.993 sentences (121.443 tokens) and covers mainly literary and journalistic texts.",
    "The Belarusian UD treebank is based on a sample of the news texts included in the Belarusian-Russian parallel subcorpus of the Russian National Corpus,",
    "UD_Bulgarian-BTB is based on the HPSG-based BulTreeBank.",
    "Catalan data from the [AnCora](http://clic.ub.edu/corpus/) corpus.",
    "Traditional Chinese Universal Dependencies Treebank annotated and converted by Google.",
    "Simplified Chinese Universal Dependencies dataset converted from the GSD (traditional) dataset with manual corrections.",
    "Classical Chinese Universal Dependencies Treebank annotated and converted by Institute for Research in Humanities, Kyoto University.",
    "UD Coptic contains manually annotated Sahidic Coptic texts, including Biblical texts, sermons, letters, and hagiography.",
    "The Croatian UD treebank is based on the extension of the SETimes-HR corpus, the [hr500k](http://hdl.handle.net/11356/1183) corpus.",
    "The UD_Czech-CAC treebank is based on the Czech Academic Corpus 2.0 (CAC) created at Charles University in Prague.",
    "The UD_Czech-CLTT treebank is based on the Czech Legal Text Treebank 2.0,",
    "FicTree is a treebank of Czech fiction, automatically converted into the UD format. The treebank was built at Charles University in Prague.",
    "The Czech-PDT UD treebank is based on the Prague Dependency Treebank – Consolidated 1.0, created at the Charles University in Prague.",
    "The Danish UD treebank is a conversion of the Danish Dependency Treebank.",
    "This corpus consists of samples from various treebanks annotated at the University of Groningen using the Alpino annotation tools and guidelines.",
    "This corpus contains sentences from the Wikipedia section of the Lassy Small Treebank.",
    "A Gold Standard Universal Dependencies Corpus for English, built over the source material of the English Web Treebank LDC2012T13 (https://catalog.ldc.upenn.edu/LDC2012T13).",
    "Universal Dependencies syntax annotations from the GUM corpus (https://gucorpling.org/gum/)",
    "UD English_LinES is the English half of the LinES Parallel Treebank with the original dependency annotation first automatically converted into Universal Dependencies and then partially reviewed. Its contents cover literature, an online manual and Europarl data.",
    "UD_English-ParTUT is a conversion of a multilingual parallel treebank developed at the University of Turin, and consisting of a variety of text genres, including talks, legal texts and Wikipedia articles, among others.",
    "UD Estonian is a converted version of the Estonian Dependency Treebank (EDT), originally annotated in the Constraint Grammar (CG) annotation scheme, and consisting of genres of fiction, newspaper texts and scientific texts. The treebank contains 30,972 trees, 437,769 tokens.",
    "UD EWT treebank consists of different genres of new media. The treebank contains 7,190 trees, 90,585 tokens.",
    "FinnTreeBank 1 consists of manually annotated grammatical examples from VISK. The UD version of FinnTreeBank 1 was converted from a native annotation model with a script and later manually revised.",
    "UD_Finnish-TDT is based on the Turku Dependency Treebank (TDT), a broad-coverage dependency treebank of general Finnish covering numerous genres. The conversion to UD was followed by extensive manual checks and corrections, and the treebank closely adheres to the UD guidelines.",
    "The UD_French-GSD was converted in 2015 from the content head version of the universal dependency treebank v2.0 (https://github.com/ryanmcd/uni-dep-tb). It is updated since 2015 independently from the previous source.",
    "UD_French-ParTUT is a conversion of a multilingual parallel treebank developed at the University of Turin, and consisting of a variety of text genres, including talks, legal texts and Wikipedia articles, among others.",
    "**UD_French-Sequoia** is an automatic conversion of the [SUD_French-Sequoia](https://github.com/surfacesyntacticud/SUD_French-Sequoia) treebank, which comes from the former corpus [French Sequoia corpus](http://deep-sequoia.inria.fr).",
    "The Galician UD treebank is based on the automatic parsing of the Galician Technical Corpus (http://sli.uvigo.gal/CTG) created at the University of Vigo by the the TALG NLP research group.",
    "The Galician-TreeGal is a treebank for Galician developed at LyS Group (Universidade da Coruña) and at CiTIUS (Universidade de Santiago de Compostela).",
    "The German UD is converted from the content head version of the [universal dependency treebank v2.0 (legacy)](https://github.com/ryanmcd/uni-dep-tb).",
    "UD German-HDT is a conversion of the Hamburg Dependency Treebank, created at the University of Hamburg through manual annotation in conjunction with a standard for morphologically and syntactically annotating sentences as well as a constraint-based parser.",
    "The UD Gothic treebank is based on the Gothic data from the PROIEL treebank, and consists of Wulfila's Bible translation.",
    "The Greek UD treebank (UD_Greek-GDT) is derived from the Greek Dependency Treebank.",
    "A Universal Dependencies Corpus for Hebrew.",
    "The Hindi UD treebank is based on the Hindi Dependency Treebank (HDTB), created at IIIT Hyderabad, India.",
    "The Hungarian UD treebank is derived from the Szeged Dependency Treebank (Vincze et al. 2010).",
    "The Indonesian-GSD treebank was originally converted from the content head version of the [universal dependency treebank v2.0 (legacy)](https://github.com/ryanmcd/uni-dep-tb) in 2015. In order to comply with the latest Indonesian annotation guidelines, the treebank has undergone a major revision between UD releases v2.8 and v2.9 (2021).",
    "A Universal Dependencies 4910-sentence treebank for modern Irish.",
    "The Italian corpus annotated according to the UD annotation scheme was obtained by conversion from ISDT (Italian Stanford Dependency Treebank), released for the dependency parsing shared task of Evalita-2014 (Bosco et al. 2014).",
    "UD_Italian-ParTUT is a conversion of a multilingual parallel treebank developed at the University of Turin, and consisting of a variety of text genres, including talks, legal texts and Wikipedia articles, among others.",
    "PoSTWITA-UD is a collection of Italian tweets annotated in Universal Dependencies that can be exploited for the training of NLP systems to enhance their performance on social media texts.",
    "TWITTIRÒ-UD is a collection of ironic Italian tweets annotated in Universal Dependencies. The treebank can be exploited for the training of NLP systems to enhance their performance on social media texts, and in particular, for irony detection purposes.",
    "The UD_Italian-VIT corpus was obtained by conversion from VIT (Venice Italian Treebank), developed at the Laboratory of Computational Linguistics of the Università Ca' Foscari in Venice (Delmonte et al. 2007; Delmonte 2009; http://rondelmo.it/resource/VIT/Browser-VIT/index.htm).",
    "This Universal Dependencies (UD) Japanese treebank is based on the definition of UD Japanese convention described in the UD documentation.  The original sentences are from Google UDT 2.0.",
    "The Google Korean Universal Dependency Treebank is first converted from the [Universal Dependency Treebank v2.0 (legacy)](https://github.com/ryanmcd/uni-dep-tb), and then enhanced by Chun et al., 2018.",
    "The KAIST Korean Universal Dependency Treebank is generated by Chun et al., 2018 from the constituency trees in the [KAIST Tree-Tagging Corpus](http://semanticweb.kaist.ac.kr/home/index.php/Corpus4).",
    "Latin data from the _Index Thomisticus_ Treebank. Data are taken from the _Index Thomisticus_ corpus by Roberto Busa SJ, which contains the complete work by Thomas Aquinas (1225–1274; Medieval Latin) and by 61 other authors related to Thomas.",
    "This Universal Dependencies Latin Treebank consists of an automatic conversion of a selection of passages from the Ancient Greek and Latin Dependency Treebank 2.1",
    "The Latin PROIEL treebank is based on the Latin data from the PROIEL treebank, and contains most of the Vulgate New Testament translations plus selections from Caesar's Gallic War, Cicero's Letters to Atticus, Palladius' Opus Agriculturae and the first book of Cicero's De officiis.",
    "Latvian UD Treebank is based on Latvian Treebank ([LVTB](http://sintakse.korpuss.lv)), being created at University of Latvia, Institute of Mathematics and Computer Science, [Artificial Intelligence Laboratory](http://ailab.lv).",
    "The Lithuanian dependency treebank ALKSNIS v3.0 (Vytautas Magnus University).",
    "Lithuanian treebank annotated manually (dependencies) using the Morphological Annotator by CCL, Vytautas Magnus University (http://tekstynas.vdu.lt/) and manual disambiguation.",
    "MUDT (Maltese Universal Dependencies Treebank) is a manually annotated treebank of Maltese, a Semitic language of Malta descended from North African Arabic with a significant amount of Italo-Romance influence.",
    "UD Marathi is a manually annotated treebank consisting primarily of stories from Wikisource, and parts of an article on Wikipedia.",
    "The Norwegian UD treebank is based on the Bokmål section of the Norwegian Dependency Treebank (NDT), which is a syntactic treebank of Norwegian. The current version of NDT has been automatically converted to the UD scheme by Ingerid Løyning Dale, Per Erik Solberg and Andre Kåsen at the Norwegian Language Bank at the National Library of Norway. This conversion builds to a large extent on previous conversions by Lilja Øvrelid at the University of Oslo.",
    "The Norwegian UD treebank is based on the Nynorsk section of the Norwegian Dependency Treebank (NDT), which is a syntactic treebank of Norwegian. NDT has been automatically converted to the UD scheme by Lilja Øvrelid at the University of Oslo.",
    "The Old Church Slavonic (OCS) UD treebank is based on canonical Old Church Slavonic data from the PROIEL and TOROT treebanks.",
    "The Persian Universal Dependency Treebank (Seraji) is based on Uppsala Persian Dependency Treebank (UPDT).",
    "The LFG Enhanced UD treebank of Polish is based on a corpus of LFG (Lexical Functional Grammar) syntactic structures generated by an LFG grammar of Polish, POLFIE, and manually disambiguated by human annotators.",
    "The Polish PDB-UD treebank is automatically converted from the Polish Dependency Bank 2.0 (PDB 2.0). Both treebanks were created at the [Institute of Computer Science, Polish Academy of Sciences](https://ipipan.waw.pl/en/) in Warsaw (Poland).",
    "This Universal Dependencies (UD) Portuguese treebank is based on the Constraint Grammar converted version of the Bosque, which is part of the Floresta Sintá(c)tica treebank. It contains both European (CETEMPúblico) and Brazilian (CETENFolha) variants.",
    "The Brazilian Portuguese UD is converted from the [Google Universal Dependency",
    "The Romanian Non-standard UD treebank (called UAIC-RoDia) is based on UAIC-RoDia Treebank. UAIC-RoDia = ISLRN 156-635-615-024-0",
    "The Romanian UD treebank (called RoRefTrees) (Barbu Mititelu et al., 2016) is the reference treebank in UD format for standard Romanian.",
    "Russian Universal Dependencies Treebank annotated and converted by Google.",
    "Russian data from the SynTagRus corpus.",
    "Universal Dependencies treebank is based on data samples extracted from Taiga Corpus and MorphoRuEval-2017 and GramEval-2020 shared tasks collections.",
    "A treebank of Scottish Gaelic based on the [Annotated Reference Corpus Of Scottish Gaelic (ARCOSG)](https://github.com/Gaelic-Algorithmic-Research-Group/ARCOSG).",
    "The Serbian UD treebank is based on the [SETimes-SR](http://hdl.handle.net/11356/1200) corpus and additional news documents from the Serbian web.",
    "The Slovak UD treebank is based on data originally annotated as part of the Slovak National Corpus, following the annotation style of the Prague Dependency Treebank.",
    "The SSJ treebank is the reference UD treebank for Slovenian, consisting of approximately 13,000 sentences and 267,097 tokens from fiction, non-fiction, periodical and Wikipedia texts in standard modern Slovenian. As of UD release 2.10 in May 2022, the original version of the SSJ UD treebank has been partially manually revised and extended with new manually annotated data.",
    "The Spoken Slovenian Treebank (SST) is a manually annotated collection of transcribed audio recordings featuring spontaneous speech in various everyday situations. It includes 344 unique speech events (documents) amounting to approximately 10 hours of speech, encompassing a total of 6,108 utterances and 98,396 tokens.",
    "Spanish data from the [AnCora](http://clic.ub.edu/corpus/) corpus.",
    "The Spanish UD is converted from the content head version of the [universal dependency treebank v2.0 (legacy)](https://github.com/ryanmcd/uni-dep-tb).",
    "UD Swedish_LinES is the Swedish half of the LinES Parallel Treebank with UD annotations. All segments are translations from English and the sources cover literary genres, online manuals and Europarl data.",
    "The Swedish-Talbanken treebank is based on Talbanken, a treebank developed at Lund University in the 1970s.",
    "The UD Tamil treebank is based on the Tamil Dependency Treebank created at the Charles University in Prague by Loganathan Ramasamy.",
    "The Telugu UD treebank is created in UD based on manual annotations of sentences from a grammar book.",
    "The UD Turkish Treebank, also called the IMST-UD Treebank, is a semi-automatic conversion of the IMST Treebank (Sulubacak&Eryiğit, 2018; Sulubacak et al., 2016).",
    "Gold standard Universal Dependencies corpus for Ukrainian, developed for UD originally, by [Institute for Ukrainian](https://mova.institute), NGO.   [[українською](https://mova.institute/золотий_стандарт)]",
    "The Urdu Universal Dependency Treebank was automatically converted from Urdu Dependency Treebank (UDTB) which is part of an ongoing effort of creating multi-layered treebanks for Hindi and Urdu.",
    "The Uyghur UD treebank is based on the Uyghur Dependency Treebank (UDT), created at the Xinjiang University, China.",
    "The Vietnamese UD treebank is a conversion of the constituent treebank created in the VLSP project (https://vlsp.hpda.vn/).",
    "UD_Wolof-WTB is a natively manual developed treebank for Wolof. Sentences were collected from encyclopedic, fictional, biographical, religious texts and news."
  )

  contributors <- c(
    "Peter Dirix, Liesbeth Augustinus, Daniel van Niekerk",
    "Giuseppe G. A. Celano, Daniel Zeman",
    "Dag Haug",
    "Daniel Zeman, Zdeněk Žabokrtský, Shadi Saleh",
    "Marat M. Yavrumyan",
    "Maria Jesus Aranzabe, Aitziber Atutxa, Kepa Bengoetxea, Arantza Diaz de Ilarraza, Iakes Goenaga, Koldo Gojenola, Larraitz Uria",
    "Olga Lyashevskaya, Angelika Peljak-Łapińska, Daria Petrova, Yana Shishkina",
    "Kiril Simov, Petya Osenova, Martin Popel",
    "Héctor Martínez Alonso, Elena Pascual, Daniel Zeman",
    "Mo Shen, Ryan McDonald, Daniel Zeman, Peng Qi",
    "Peng Qi, Koichi Yasuoka",
    "Koichi Yasuoka, Christian Wittern, Tomohiko Morioka, Takumi Ikeda, Naoki Yamazaki, Yoshihiro Nikaido, Shingo Suzuki, Shigeki Moro, Yuan Li, Hiroyuki Shirasu, Kazunori Fujita",
    "Mitchell Abrams, Elizabeth Davidson, Amir Zeldes",
    "Željko Agić, Nikola Ljubešić, Daniel Zeman",
    "Barbora Hladká, Daniel Zeman",
    "Barbora Hladká, Daniel Zeman, Martin Popel",
    "Tomáš Jelínek, Daniel Zeman",
    "Daniel Zeman, Jan Hajič",
    "Anders Johannsen, Héctor Martínez Alonso, Barbara Plank",
    "Daniel Zeman, Zdeněk Žabokrtský, Gosse Bouma, Gertjan van Noord",
    "Gosse Bouma, Gertjan van Noord",
    "Natalia Silveira, Timothy Dozat, Christopher Manning, Sebastian Schuster, Ethan Chi, John Bauer, Miriam Connor, Marie-Catherine de Marneffe, Nathan Schneider, Sam Bowman, Hanzhi Zhu, Daniel Galbraith, John Bauer",
    "Siyao Peng, Amir Zeldes",
    "Lars Ahrenberg",
    "Cristina Bosco, Manuela Sanguinetti",
    "Kadri Muischnek, Kaili Müürisep, Tiina Puolakainen, Andriela Rääbis, Liisi Torga",
    "Kadri Muischnek, Kaili Müürisep, Tiina Puolakainen, Dage Särg, Sandra Eiche, Andriela Rääbis",
    "Jussi Piitulainen, Hanna Nurmi, Jack Rueter",
    "Filip Ginter, Jenna Kanerva, Veronika Laippala, Niko Miekka, Anna Missilä, Stina Ojala, Sampo Pyysalo",
    "Marie-Catherine de Marneffe, Bruno Guillaume, Ryan McDonald, Alane Suhr, Joakim Nivre, Matias Grioni, Carly Dickerson, Guy Perrier",
    "Cristina Bosco, Manuela Sanguinetti",
    "Marie Candito, Djamé Seddah, Guy Perrier, Bruno Guillaume",
    "Xavier Gómez Guinovart",
    "Marcos Garcia, Xulia Sánchez-Rodríguez",
    "Slav Petrov, Wolfgang Seeker, Ryan McDonald, Joakim Nivre, Daniel Zeman, Adriane Boyd",
    "Emanuel Borges Völker, Felix Hennig, Arne Köhn, Maximilan Wendt, Verena Blaschke, Nina Böbel, Leonie Weissweiler",
    "Dag Haug",
    "Prokopis Prokopidis",
    "Yoav Goldberg, Reut Tsarfaty, Amir More, Shoval Sadde, Victoria Basmov, Yuval Pinter",
    "Riyaz Ahmad Bhat, Daniel Zeman",
    "Richárd Farkas, Katalin Simkó, Zsolt Szántó, Viktor Varga, Veronika Vincze",
    "Ryan McDonald, Joakim Nivre, Daniel Zeman, Septina Dian Larasati, Ika Alfina",
    "Teresa Lynn, Jennifer Foster, Sarah McGuinness, Abigail Walsh, Jason Phelan, Kevin Scannell",
    "Cristina Bosco, Alessandro Lenci, Simonetta Montemagni, Maria Simi",
    "Cristina Bosco, Manuela Sanguinetti",
    "Cristina Bosco, Manuela Sanguinetti",
    "Alessandra T. Cignarella, Cristina Bosco, Manuela Sanguinetti",
    "Fabio Tamburini, Maria Simi, Cristina Bosco",
    "Mai Omura, Yusuke Miyao, Hiroshi Kanayama, Hiroshi Matsuda, Aya Wakasa, Kayo Yamashita, Masayuki Asahara, Takaaki Tanaka, Yugo Murawaki, Yuji Matsumoto, Shinsuke Mori, Sumire Uematsu, Ryan McDonald, Joakim Nivre, Daniel Zeman",
    "Ryan McDonald, Joakim Nivre, Daniel Zeman, Jinho Choi, Na-Rae Han, Jena Hwang, Jayeol Chun",
    "Jinho Choi, Na-Rae Han, Jena Hwang, Jayeol Chun",
    "Marco Passarotti, Marinella Testori, Daniel Zeman, Berta González Saavedra, Flavio Massimiliano Cecchini",
    "Giuseppe G. A. Celano, Daniel Zeman, Federica Gamba",
    "Dag Haug",
    "Lauma Pretkalniņa, Laura Rituma, Gunta Nešpore-Bērzkalne, Baiba Saulīte, Artūrs Znotiņš, Normunds Grūzītis",
    "Andrius Utka, Erika Rimkutė, Agnė Bielinskienė, Jolanta Kovalevskaitė, Loïc Boizou, Gabrielė Aleksandravičiūtė, Kristina Brokaitė, Daniel Zeman, Natalia Perkova, Bernadeta Griciūtė",
    "Olga Lyashevskaya, Dmitri Sitchinava",
    "Slavomír Čéplö, Daniel Zeman",
    "Vinit Ravishankar",
    "Lilja Øvrelid, Fredrik Jørgensen, Petter Hohle, Ingerid Løyning Dale, Per Erik Solberg, Andre Kåsen",
    "Lilja Øvrelid, Fredrik Jørgensen, Petter Hohle, Ingerid Løyning Dale, Per Erik Solberg, Andre Kåsen",
    "Dag Haug",
    "Mojgan Seraji, Filip Ginter, Joakim Nivre, Martin Popel, Daniel Zeman",
    "Agnieszka Patejuk, Adam Przepiórkowski",
    "Alina Wróblewska, Daniel Zeman, Jan Mašek, Rudolf Rosa",
    "Alexandre Rademaker, Cláudia Freitas, Elvis de Souza, Aline Silveira, Tatiana Cavalcanti, Wograine Evelyn, Luisa Rocha, Isabela Soares-Bastos, Eckhard Bick, Fabricio Chalub, Guilherme Paulino-Passos, Livy Real, Valeria de Paiva, Daniel Zeman, Martin Popel, David Mareček, Natalia Silveira, André Martins",
    "Alexandre Rademaker, Ryan McDonald, Joakim Nivre, Daniel Zeman, Fabricio Chalub, Carlos Ramisch, Juan Belieni, Vanessa Berwanger Wille, Rodrigo Pintucci",
    "Cătălina Mărănduc, Cenel-Augusto Perez, Victoria Bobicev, Cătălin Mititelu, Florinel Hociung, Valentin Roșca, Roman Untilov, Petru Rebeja",
    "Verginica Barbu Mititelu, Elena Irimia, Cenel-Augusto Perez, Radu Ion, Radu Simionescu, Martin Popel",
    "Ryan McDonald, Vitaly Nikolaev, Olga Lyashevskaya",
    "Kira Droganova, Olga Lyashevskaya, Daniel Zeman",
    "Olga Lyashevskaya, Olga Rudina, Natalia Vlasova, Anna Zhuravleva",
    "Colin Batchelor",
    "Tanja Samardžić, Nikola Ljubešić",
    "Katarína Gajdošová, Mária Šimková, Daniel Zeman",
    "Kaja Dobrovoljc, Tomaž Erjavec, Simon Krek",
    "Kaja Dobrovoljc, Joakim Nivre",
    "Héctor Martínez Alonso, Daniel Zeman",
    "Miguel Ballesteros, Héctor Martínez Alonso, Ryan McDonald, Elena Pascual, Natalia Silveira, Daniel Zeman, Joakim Nivre",
    "Lars Ahrenberg",
    "Joakim Nivre, Aaron Smith, Victor Norrman",
    "Loganathan Ramasamy, Daniel Zeman",
    "Taraka Rama, Sowmya Vajjala",
    "Utku Türk, Şaziye Betül Özateş, Büşra Marşan, Salih Furkan Akkurt, Çağrı Çöltekin, Gülşen Cebiroğlu Eryiğit, Memduh Gökırmak, Hüner Kaşıkara, Umut Sulubacak, Francis Tyers",
    "Natalia Kotsyba, Bohdan Moskalevskyi, Mykhailo Romanenko",
    "Riyaz Ahmad Bhat, Daniel Zeman",
    "Marhaba Eli, Daniel Zeman, Francis Tyers",
    "Lương Nguyễn Thị, Linh Hà Mỹ, Phương Lê Hồng, Huyền Nguyễn Thị Minh",
    "Bamba Dione"
  )

  file <- c(
    "af_afribooms",
    "grc_perseus",
    "grc_proiel",
    "ar_padt",
    "hy_armtdp",
    "eu_bdt",
    "be_hse",
    "bg_btb",
    "ca_ancora",
    "zh_gsd",
    "zh_gsdsimp",
    "lzh_kyoto",
    "cop_scriptorium",
    "hr_set",
    "cs_cac",
    "cs_cltt",
    "cs_fictree",
    "cs_pdt",
    "da_ddt",
    "nl_alpino",
    "nl_lassysmall",
    "en_ewt",
    "en_gum",
    "en_lines",
    "en_partut",
    "et_edt",
    "et_ewt",
    "fi_ftb",
    "fi_tdt",
    "fr_gsd",
    "fr_partut",
    "fr_sequoia",
    "gl_ctg",
    "gl_treegal",
    "de_gsd",
    "de_hdt",
    "got_proiel",
    "el_gdt",
    "he_htb",
    "hi_hdtb",
    "hu_szeged",
    "id_gsd",
    "ga_idt",
    "it_isdt",
    "it_partut",
    "it_postwita",
    "it_twittiro",
    "it_vit",
    "ja_gsd",
    "ko_gsd",
    "ko_kaist",
    "la_ittb",
    "la_perseus",
    "la_proiel",
    "lv_lvtb",
    "lt_alksnis",
    "lt_hse",
    "mt_mudt",
    "mr_ufal",
    "no_bokmaal",
    "no_nynorsk",
    "cu_proiel",
    "fa_seraji",
    "pl_lfg",
    "pl_pdb",
    "pt_bosque",
    "pt_gsd",
    "ro_nonstandard",
    "ro_rrt",
    "ru_gsd",
    "ru_syntagrus",
    "ru_taiga",
    "gd_arcosg",
    "sr_set",
    "sk_snk",
    "sl_ssj",
    "sl_sst",
    "es_ancora",
    "es_gsd",
    "sv_lines",
    "sv_talbanken",
    "ta_ttb",
    "te_mtg",
    "tr_imst",
    "uk_iu",
    "ur_udtb",
    "ug_udt",
    "vi_vtb",
    "wo_wtb"
  )

  hub_link <- paste0(
    "https://universaldependencies.org/treebanks/",
    file,
    "/index.html"
  )

  tokens <- c(
    "22369",
    "202989",
    "214005",
    "93712",
    "52220",
    "121443",
    "305417",
    "156149",
    "537767",
    "123291",
    "123291",
    "433168",
    "26837",
    "2247925",
    "494142",
    "36007",
    "166602",
    "1526323",
    "506231",
    "208745",
    "297486",
    "251536",
    "208446",
    "93200",
    "49601",
    "438312",
    "50473",
    "159314",
    "202209",
    "389367",
    "27638",
    "68593",
    "126011",
    "23479",
    "287721",
    "3399390",
    "87266",
    "61773",
    "114648",
    "351704",
    "1182501",
    "119830",
    "115990",
    "278461",
    "51614",
    "119334",
    "28384",
    "259625",
    "193654",
    "80322",
    "350090",
    "450480",
    "28868",
    "328476",
    "328305",
    "70049",
    "5356",
    "44162",
    "13089",
    "310221",
    "301353",
    "553686",
    "151625",
    "130967",
    "347319",
    "210958",
    "296169",
    "1896343",
    "218522",
    "97994",
    "1517881",
    "197001",
    "86089",
    "97673",
    "365490",
    "267097",
    "98393",
    "555670",
    "423346",
    "90961",
    "96859",
    "8635",
    "6465",
    "56422",
    "122934",
    "138077",
    "40236",
    "58069",
    "42832"
  )

  words <- c(
    "25017",
    "202989",
    "214005",
    "94390",
    "52585",
    "121443",
    "305417",
    "156149",
    "553638",
    "123291",
    "123291",
    "433168",
    "57098",
    "2252453",
    "495219",
    "36023",
    "167226",
    "1529074",
    "506232",
    "208746",
    "297486",
    "254865",
    "212035",
    "94217",
    "49632",
    "438313",
    "50486",
    "159612",
    "202453",
    "400391",
    "28576",
    "70545",
    "138837",
    "25548",
    "292769",
    "3455580",
    "88934",
    "63441",
    "160195",
    "351704",
    "1183884",
    "122019",
    "115990",
    "298375",
    "55558",
    "124515",
    "29602",
    "280153",
    "193654",
    "80322",
    "350090",
    "450517",
    "29221",
    "328476",
    "328305",
    "70049",
    "5356",
    "44162",
    "13089",
    "310221",
    "301353",
    "553686",
    "152920",
    "130967",
    "349978",
    "227827",
    "318666",
    "1896343",
    "218522",
    "97994",
    "1517881",
    "197001",
    "89958",
    "97673",
    "365490",
    "267097",
    "98393",
    "568249",
    "431600",
    "90961",
    "96859",
    "9581",
    "6465",
    "58096",
    "122983",
    "138077",
    "40236",
    "58069",
    "44258"
  )

  sentences <- c(
    "1975",
    "13919",
    "17082",
    "4800",
    "2500",
    "8993",
    "25231",
    "11138",
    "16678",
    "4997",
    "4997",
    "86239",
    "2203",
    "127794",
    "24709",
    "1121",
    "12760",
    "87907",
    "30723",
    "13603",
    "17120",
    "16622",
    "12146",
    "5243",
    "2090",
    "30930",
    "2829",
    "18723",
    "15136",
    "16342",
    "1020",
    "3099",
    "3993",
    "1000",
    "15590",
    "189928",
    "4328",
    "2521",
    "6143",
    "16649",
    "53564",
    "5598",
    "4910",
    "14167",
    "2090",
    "6712",
    "1424",
    "10087",
    "8100",
    "6339",
    "27363",
    "26977",
    "2273",
    "19387",
    "19367",
    "3642",
    "263",
    "2074",
    "1144",
    "20044",
    "17575",
    "40085",
    "5997",
    "17246",
    "22152",
    "9357",
    "12020",
    "116324",
    "9524",
    "5030",
    "87336",
    "17872",
    "4741",
    "4384",
    "19543",
    "13435",
    "6108",
    "17662",
    "16014",
    "5243",
    "6026",
    "600",
    "1328",
    "5635",
    "7092",
    "5130",
    "3456",
    "3323",
    "2107"
  )

  folder <- paste0("UD_", language_name, "-", treebank) # Generazione dinamica

  chapter <- c(
    "Hoofstuk", # Afrikaans
    "κεφάλαιο", # Ancient Greek
    "κεφάλαιο", # Ancient Greek (duplicated)
    "فصل", # Arabic
    "Գլուխ", # Armenian
    "Kapitulua", # Basque
    "Глава", # Belarusian
    "Глава", # Bulgarian
    "Capítol", # Catalan
    "章节", # Chinese (Simplified)
    "章", # Chinese (Traditional)
    "章节", # Classical Chinese
    "ⲕⲁⲡⲓⲧⲓⲗⲟⲛ", # Coptic
    "Poglavlje", # Croatian
    "Kapitola", # Czech
    "Kapitola", # Czech (duplicated)
    "Kapitola", # Czech (duplicated)
    "Kapitola", # Czech (duplicated)
    "Kapitel", # Danish
    "Hoofdstuk", # Dutch
    "Hoofdstuk", # Dutch (duplicated)
    "Chapter", # English
    "Chapter", # English (duplicated)
    "Chapter", # English (duplicated)
    "Chapter", # English (duplicated)
    "Peatükk", # Estonian
    "Peatükk", # Estonian (duplicated)
    "Luku", # Finnish
    "Luku", # Finnish (duplicated)
    "Chapitre", # French
    "Chapitre", # French (duplicated)
    "Chapitre", # French (duplicated)
    "Capítulo", # Galician
    "Capítulo", # Galician (duplicated)
    "Kapitel", # German
    "Kapitel", # German (duplicated)
    "𐍉𐌼𐌿𐌽𐌳𐌰𐌽𐌾𐌾𐌰", # Gothic
    "Κεφάλαιο", # Greek
    "פרק", # Hebrew
    "अध्याय", # Hindi
    "Fejezet", # Hungarian
    "Bab", # Indonesian
    "Caibidil", # Irish
    "Capitolo", # Italian
    "Capitolo", # Italian (duplicated)
    "Capitolo", # Italian (duplicated)
    "Capitolo", # Italian (duplicated)
    "Capitolo", # Italian (duplicated)
    "章", # Japanese
    "장", # Korean
    "장", # Korean (duplicated)
    "Capitulum", # Latin
    "Capitulum", # Latin (duplicated)
    "Capitulum", # Latin (duplicated)
    "Nodaļa", # Latvian
    "Skyrius", # Lithuanian
    "Skyrius", # Lithuanian (duplicated)
    "Kapitel", # Maltese
    "अध्याय", # Marathi
    "Kapittel", # Norwegian
    "Kapittel", # Norwegian (duplicated)
    "Глава", # Old Church Slavonic
    "فصل", # Persian
    "Rozdział", # Polish
    "Rozdział", # Polish (duplicated)
    "Capítulo", # Portuguese
    "Capítulo", # Portuguese (duplicated)
    "Capitol", # Romanian
    "Capitol", # Romanian (duplicated)
    "Глава", # Russian
    "Глава", # Russian (duplicated)
    "Глава", # Russian (duplicated)
    "Caibideil", # Scottish Gaelic
    "Глава", # Serbian
    "Kapitola", # Slovak
    "Poglavje", # Slovenian
    "Poglavje", # Slovenian (duplicated)
    "Capítulo", # Spanish
    "Capítulo", # Spanish (duplicated)
    "Kapitel", # Swedish
    "Kapitel", # Swedish (duplicated)
    "அத்தியாயம்", # Tamil
    "అధ్యాయం", # Telugu
    "Bölüm", # Turkish
    "Розділ", # Ukrainian
    "باب", # Urdu
    "باب", # Uyghur
    "Chương", # Vietnamese
    "Kàll" # Wolof
  )

  flag <- c(
    "AF.svg",
    "GRC.svg",
    "GRC.svg",
    "AR.svg",
    "HY.svg",
    "EU.svg",
    "BE.svg",
    "BG.svg",
    "CA.svg",
    "ZH.svg",
    "ZH.svg",
    "LZH.svg",
    "COP.svg",
    "HR.svg",
    "CS.svg",
    "CS.svg",
    "CS.svg",
    "CS.svg",
    "DA.svg",
    "NL.svg",
    "NL.svg",
    "EN.svg",
    "EN.svg",
    "EN.svg",
    "EN.svg",
    "ET.svg",
    "ET.svg",
    "FI.svg",
    "FI.svg",
    "FR.svg",
    "FR.svg",
    "FR.svg",
    "GL.svg",
    "GL.svg",
    "DE.svg",
    "DE.svg",
    "GOT.svg",
    "EL.svg",
    "HE.svg",
    "HI.svg",
    "HU.svg",
    "ID.svg",
    "GA.svg",
    "IT.svg",
    "IT.svg",
    "IT.svg",
    "IT.svg",
    "IT.svg",
    "JA.svg",
    "KO.svg",
    "KO.svg",
    "LA.svg",
    "LA.svg",
    "LA.svg",
    "LV.svg",
    "LT.svg",
    "LT.svg",
    "MT.svg",
    "MR.svg",
    "NO.svg",
    "NO.svg",
    "CU.svg",
    "FA.svg",
    "PL.svg",
    "PL.svg",
    "PT.svg",
    "PT.svg",
    "RO.svg",
    "RO.svg",
    "RU.svg",
    "RU.svg",
    "RU.svg",
    "GD.svg",
    "SR.svg",
    "SK.svg",
    "SL.svg",
    "SL.svg",
    "ES.svg",
    "ES.svg",
    "SV.svg",
    "SV.svg",
    "TA.svg",
    "TE.svg",
    "TR.svg",
    "UK.svg",
    "UR.svg",
    "UG.svg",
    "VI.svg",
    "WO.svg"
  )

  ranking <- c(
    21,
    22,
    23,
    24,
    25,
    26,
    27,
    28,
    29,
    19,
    20,
    30,
    31,
    32,
    33,
    34,
    35,
    36,
    37,
    38,
    39,
    2,
    1,
    3,
    4,
    40,
    41,
    42,
    43,
    10,
    11,
    12,
    44,
    45,
    16,
    15,
    46,
    47,
    48,
    49,
    50,
    51,
    52,
    5,
    6,
    7,
    8,
    9,
    53,
    54,
    55,
    56,
    57,
    58,
    59,
    60,
    61,
    62,
    63,
    64,
    65,
    66,
    67,
    68,
    69,
    17,
    18,
    70,
    71,
    72,
    73,
    74,
    75,
    76,
    77,
    78,
    79,
    13,
    14,
    80,
    81,
    82,
    83,
    84,
    85,
    86,
    87,
    88,
    89
  )

  df <- data.frame(
    language_name = language_name,
    treebank = treebank,
    contributors = contributors,
    description = description,
    file = file,
    folder = folder,
    hub_page_link = hub_link,
    flag = flag,
    tokens = tokens,
    words = words,
    sentences = sentences,
    chapter = toupper(chapter),
    index = ranking
  )
  # df <- df[index,]
  df <- df %>%
    arrange(index) %>%
    select(-index) %>%
    filter(!file %in% c("gl_treegal", "la_perseus"))

  return(df)
}

model_accuracy <- function() {
  # Creazione dei vettori stringa per ciascuna colonna
  Language <- c(
    "Afrikaans",
    "Arabic",
    "Belarusian",
    "Bulgarian",
    "Catalan",
    "Coptic",
    "Czech",
    "Czech",
    "Czech",
    "Czech",
    "Old Church Slavonic",
    "Danish",
    "German",
    "Greek",
    "English",
    "English",
    "English",
    "English",
    "Spanish",
    "Spanish",
    "Estonian",
    "Estonian",
    "Basque",
    "Persian",
    "Finnish",
    "Finnish",
    "French",
    "French",
    "French",
    "Irish",
    "Scottish Gaelic",
    "Galician",
    "Gothic",
    "Ancient Greek",
    "Ancient Greek",
    "Hebrew",
    "Hindi",
    "Croatian",
    "Hungarian",
    "Armenian",
    "Indonesian",
    "Italian",
    "Italian",
    "Italian",
    "Italian",
    "Italian",
    "Japanese",
    "Korean",
    "Korean",
    "Latin",
    "Latin",
    "Lithuanian",
    "Lithuanian",
    "Latvian",
    "Classical Chinese",
    "Marathi",
    "Maltese",
    "Dutch",
    "Dutch",
    "Norwegian",
    "Norwegian",
    "Polish",
    "Polish",
    "Portuguese",
    "Portuguese",
    "Romanian",
    "Romanian",
    "Russian",
    "Russian",
    "Russian",
    "Slovak",
    "Slovenian",
    "Slovenian",
    "Serbian",
    "Swedish",
    "Swedish",
    "Tamil",
    "Telugu",
    "Turkish",
    "Uyghur",
    "Ukrainian",
    "Urdu",
    "Vietnamese",
    "Wolof",
    "Chinese",
    "Chinese",
    "German"
  )

  Treebank <- c(
    "AfriBooms",
    "PADT",
    "HSE",
    "BTB",
    "AnCora",
    "Scriptorium",
    "CAC",
    "CLTT",
    "FicTree",
    "PDT",
    "PROIEL",
    "DDT",
    "GSD",
    "GDT",
    "EWT",
    "GUM",
    "LinES",
    "ParTUT",
    "AnCora",
    "GSD",
    "EDT",
    "EWT",
    "BDT",
    "Seraji",
    "FTB",
    "TDT",
    "GSD",
    "ParTUT",
    "Sequoia",
    "IDT",
    "ARCOSG",
    "CTG",
    "PROIEL",
    "Perseus",
    "PROIEL",
    "HTB",
    "HDTB",
    "SET",
    "Szeged",
    "ArmTDP",
    "GSD",
    "ISDT",
    "ParTUT",
    "PoSTWITA",
    "TWITTIRO",
    "VIT",
    "GSD",
    "GSD",
    "Kaist",
    "ITTB",
    "PROIEL",
    "ALKSNIS",
    "HSE",
    "LVTB",
    "Kyoto",
    "UFAL",
    "MUDT",
    "Alpino",
    "LassySmall",
    "Bokmaal",
    "Nynorsk",
    "LFG",
    "PDB",
    "Bosque",
    "GSD",
    "Nonstandard",
    "RRT",
    "GSD",
    "SynTagRus",
    "Taiga",
    "SNK",
    "SSJ",
    "SST",
    "SET",
    "LinES",
    "Talbanken",
    "TTB",
    "MTG",
    "IMST",
    "UDT",
    "IU",
    "UDTB",
    "VTB",
    "WTB",
    "GSD",
    "GSDSimp",
    "HDT"
  )

  Words <- c(
    99.93,
    94.51,
    99.28,
    99.92,
    99.94,
    75.38,
    99.96,
    99.37,
    99.99,
    99.91,
    100.00,
    99.93,
    99.56,
    99.89,
    99.14,
    99.57,
    99.94,
    99.78,
    99.94,
    99.72,
    99.88,
    98.95,
    99.98,
    99.65,
    99.99,
    99.68,
    98.89,
    99.41,
    99.09,
    99.67,
    97.39,
    99.14,
    100.00,
    99.98,
    100.00,
    85.08,
    99.99,
    99.91,
    99.69,
    99.38,
    99.45,
    99.80,
    99.70,
    99.36,
    99.02,
    99.75,
    95.77,
    99.85,
    99.99,
    99.97,
    99.99,
    99.95,
    97.65,
    99.23,
    97.70,
    95.27,
    99.79,
    99.59,
    99.83,
    99.82,
    99.90,
    99.86,
    99.85,
    99.67,
    99.27,
    98.88,
    99.68,
    99.53,
    99.57,
    98.74,
    100.00,
    99.89,
    100.00,
    100.00,
    99.96,
    99.91,
    94.93,
    100.00,
    97.60,
    99.74,
    99.71,
    99.99,
    85.11,
    99.16,
    89.89,
    89.05,
    99.9
  )

  Sentences <- c(
    99.65,
    81.43,
    84.80,
    94.67,
    99.08,
    30.20,
    99.76,
    93.79,
    97.56,
    91.91,
    34.43,
    89.92,
    78.69,
    89.51,
    86.17,
    95.07,
    87.88,
    99.02,
    97.92,
    94.54,
    89.81,
    75.16,
    99.42,
    98.00,
    86.93,
    85.59,
    93.45,
    98.64,
    85.01,
    97.57,
    56.07,
    94.80,
    28.25,
    98.73,
    45.89,
    99.69,
    97.69,
    93.54,
    94.48,
    97.28,
    90.47,
    96.59,
    99.02,
    24.37,
    28.57,
    96.73,
    99.45,
    93.63,
    100.00,
    92.53,
    28.48,
    87.75,
    92.04,
    99.06,
    42.03,
    81.19,
    82.32,
    89.99,
    81.93,
    97.34,
    92.74,
    99.83,
    96.06,
    89.56,
    87.99,
    96.92,
    94.01,
    97.41,
    97.55,
    84.37,
    77.73,
    99.07,
    93.73,
    93.84,
    86.83,
    94.20,
    95.08,
    98.98,
    97.51,
    82.17,
    96.71,
    97.12,
    97.02,
    90.27,
    98.80,
    98.80,
    92.6
  )

  UPOS <- c(
    95.79,
    88.91,
    96.07,
    98.01,
    98.22,
    72.69,
    98.56,
    94.87,
    97.60,
    98.63,
    91.36,
    95.24,
    64.12,
    95.13,
    94.10,
    95.77,
    95.07,
    94.30,
    98.25,
    95.49,
    95.45,
    88.85,
    92.43,
    96.02,
    92.01,
    94.28,
    96.24,
    95.26,
    96.42,
    93.43,
    91.38,
    96.23,
    93.73,
    81.44,
    95.34,
    80.84,
    95.84,
    96.67,
    90.96,
    92.39,
    93.22,
    97.24,
    96.73,
    94.25,
    90.37,
    96.38,
    93.50,
    91.40,
    92.95,
    98.42,
    94.53,
    90.37,
    71.39,
    95.23,
    87.70,
    78.79,
    93.79,
    94.18,
    94.79,
    96.39,
    95.92,
    96.86,
    97.30,
    96.12,
    95.58,
    94.94,
    96.91,
    95.00,
    97.42,
    93.42,
    92.98,
    96.78,
    94.83,
    97.36,
    94.67,
    95.71,
    78.41,
    90.29,
    88.85,
    88.80,
    94.85,
    91.53,
    76.10,
    91.73,
    82.84,
    82.28,
    97.8
  )

  XPOS <- c(
    91.39,
    82.35,
    95.40,
    94.72,
    95.04,
    72.55,
    90.88,
    82.33,
    90.26,
    93.12,
    91.63,
    99.92,
    64.29,
    95.13,
    93.23,
    95.32,
    94.08,
    93.86,
    94.92,
    99.72,
    96.80,
    91.47,
    99.98,
    95.78,
    90.74,
    95.43,
    98.89,
    94.69,
    99.09,
    92.20,
    83.27,
    95.80,
    94.20,
    71.34,
    95.65,
    80.84,
    94.91,
    89.99,
    99.69,
    99.38,
    91.90,
    97.11,
    96.37,
    93.88,
    89.60,
    95.37,
    92.80,
    81.07,
    80.41,
    93.26,
    94.73,
    81.26,
    70.36,
    87.53,
    87.29,
    95.27,
    93.49,
    91.51,
    92.21,
    97.70,
    97.30,
    87.18,
    88.49,
    99.67,
    82.76,
    90.06,
    95.94,
    94.74,
    99.57,
    98.74,
    77.04,
    90.26,
    88.51,
    90.76,
    91.95,
    93.90,
    76.99,
    90.29,
    88.54,
    90.12,
    84.03,
    89.00,
    75.07,
    91.42,
    83.56,
    82.98,
    97.4
  )

  UFeats <- c(
    93.67,
    82.44,
    88.30,
    95.77,
    97.68,
    72.88,
    89.58,
    82.64,
    90.96,
    93.21,
    82.50,
    94.48,
    53.65,
    89.96,
    94.37,
    95.10,
    94.94,
    94.21,
    97.94,
    95.97,
    93.71,
    86.94,
    87.17,
    95.92,
    92.21,
    90.54,
    95.34,
    92.58,
    94.91,
    84.15,
    85.18,
    98.92,
    86.76,
    84.62,
    87.71,
    78.57,
    90.17,
    90.59,
    87.20,
    83.70,
    94.17,
    97.12,
    96.40,
    94.26,
    90.00,
    96.33,
    95.75,
    99.50,
    99.99,
    93.98,
    86.89,
    82.42,
    66.42,
    91.82,
    88.09,
    70.06,
    99.79,
    93.28,
    93.95,
    95.06,
    94.93,
    87.48,
    88.84,
    94.07,
    91.78,
    88.31,
    96.09,
    85.57,
    90.50,
    86.33,
    79.32,
    90.53,
    88.67,
    91.10,
    90.11,
    94.18,
    79.37,
    98.89,
    84.31,
    83.63,
    83.67,
    80.26,
    85.11,
    90.89,
    88.57,
    87.79,
    91.4
  )

  AllTags <- c(
    91.26,
    82.16,
    87.69,
    94.39,
    94.83,
    71.87,
    89.45,
    82.23,
    90.14,
    92.83,
    80.73,
    93.22,
    50.34,
    88.75,
    91.61,
    93.69,
    91.84,
    92.45,
    94.54,
    93.55,
    92.15,
    83.72,
    84.68,
    95.31,
    89.00,
    89.50,
    94.51,
    91.12,
    94.17,
    81.76,
    82.49,
    95.38,
    84.55,
    71.24,
    86.28,
    77.75,
    87.68,
    89.95,
    85.84,
    82.79,
    86.65,
    96.33,
    95.19,
    92.50,
    87.00,
    94.18,
    92.30,
    78.74,
    80.41,
    92.63,
    85.59,
    80.88,
    60.98,
    87.48,
    84.79,
    67.88,
    93.20,
    90.78,
    91.41,
    93.74,
    93.29,
    86.56,
    87.88,
    92.66,
    82.58,
    87.67,
    95.83,
    84.50,
    90.11,
    85.42,
    76.77,
    89.80,
    87.49,
    90.73,
    86.66,
    92.43,
    72.93,
    90.29,
    81.58,
    76.78,
    82.86,
    75.43,
    75.04,
    88.62,
    82.30,
    81.76,
    91.0
  )

  Lemma <- c(
    96.40,
    87.36,
    88.77,
    92.88,
    98.19,
    73.30,
    96.24,
    92.93,
    95.60,
    97.50,
    81.73,
    94.46,
    86.07,
    92.74,
    95.15,
    96.55,
    95.78,
    94.76,
    98.17,
    94.08,
    87.93,
    84.30,
    91.09,
    92.74,
    85.76,
    84.52,
    96.78,
    95.84,
    96.60,
    91.96,
    91.33,
    95.18,
    89.51,
    79.90,
    90.77,
    79.01,
    98.05,
    93.30,
    87.29,
    90.83,
    87.32,
    96.98,
    96.35,
    93.84,
    89.43,
    96.52,
    94.21,
    86.96,
    88.12,
    98.04,
    91.73,
    86.89,
    68.95,
    93.35,
    97.46,
    83.15,
    99.79,
    90.82,
    92.16,
    96.04,
    95.47,
    92.56,
    94.28,
    96.50,
    91.72,
    91.61,
    95.16,
    89.90,
    95.24,
    89.54,
    83.32,
    95.04,
    94.84,
    93.40,
    94.50,
    95.13,
    82.41,
    100.00,
    85.77,
    83.47,
    91.39,
    91.47,
    83.66,
    92.81,
    89.83,
    89.00,
    92.3
  )

  UAS <- c(
    82.55,
    67.64,
    77.08,
    87.98,
    86.97,
    50.32,
    84.81,
    72.06,
    84.95,
    83.49,
    66.17,
    77.57,
    71.43,
    83.36,
    81.03,
    81.52,
    81.11,
    82.15,
    85.23,
    83.40,
    77.08,
    66.96,
    74.57,
    79.98,
    77.89,
    76.79,
    86.39,
    85.33,
    84.69,
    80.52,
    74.56,
    75.72,
    70.88,
    58.70,
    68.94,
    59.74,
    90.44,
    82.08,
    70.59,
    75.17,
    80.88,
    87.46,
    86.26,
    72.70,
    70.77,
    81.76,
    84.20,
    60.94,
    72.96,
    79.31,
    63.36,
    65.80,
    43.34,
    78.23,
    65.90,
    62.55,
    75.31,
    78.21,
    80.11,
    85.36,
    83.85,
    90.64,
    84.74,
    83.83,
    82.18,
    81.26,
    82.69,
    81.58,
    83.19,
    72.77,
    78.84,
    84.17,
    70.58,
    84.90,
    81.31,
    81.51,
    59.66,
    87.93,
    61.37,
    67.06,
    77.54,
    81.25,
    47.83,
    75.18,
    58.96,
    58.27,
    77.0
  )

  LAS <- c(
    77.97,
    62.60,
    73.45,
    83.52,
    83.75,
    48.26,
    81.13,
    67.99,
    80.79,
    80.18,
    59.74,
    73.77,
    65.53,
    79.58,
    77.64,
    78.18,
    76.60,
    78.75,
    81.52,
    79.41,
    73.13,
    60.45,
    69.35,
    75.19,
    72.65,
    72.74,
    83.00,
    82.11,
    81.63,
    72.83,
    68.31,
    72.53,
    63.47,
    51.74,
    63.73,
    56.38,
    85.86,
    77.39,
    64.95,
    67.58,
    74.29,
    84.47,
    83.08,
    67.74,
    64.11,
    77.58,
    82.36,
    51.64,
    61.60,
    75.58,
    57.80,
    59.65,
    31.05,
    73.52,
    60.15,
    51.15,
    68.95,
    73.20,
    75.79,
    82.30,
    80.62,
    86.73,
    80.83,
    79.97,
    78.52,
    75.55,
    77.96,
    76.41,
    79.37,
    67.12,
    74.61,
    80.51,
    65.41,
    80.63,
    76.54,
    77.23,
    49.62,
    75.45,
    52.36,
    53.01,
    72.78,
    74.23,
    40.71,
    69.40,
    55.27,
    54.47,
    81.4
  )

  df <- data.frame(
    language_name = gsub(" ", "_", tolower(Language)),
    treebank = Treebank,
    Words = Words,
    Sentences = Sentences,
    UPOS = UPOS,
    XPOS = XPOS,
    UFeats = UFeats,
    AllTags = AllTags,
    Lemma = Lemma,
    UAS = UAS,
    LAS = LAS,
    stringsAsFactors = FALSE # Per mantenere le colonne di testo come stringhe
  )

  return(df)
}

