# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_chisq <- function(T1, T2, Tsum) {
    .Call('_tall_cpp_chisq', PACKAGE = 'tall', T1, T2, Tsum)
}

cpp_split_tab <- function(tab, indices) {
    .Call('_tall_cpp_split_tab', PACKAGE = 'tall', tab, indices)
}

cpp_switch_docs <- function(tab1, tab2) {
    .Call('_tall_cpp_switch_docs', PACKAGE = 'tall', tab1, tab2)
}

extract_np_cpp <- function(sent_id, token_id, head_token_id, dep_rel, upos, terms, ngram_max = 5L, max_gap = 3L) {
    .Call('_tall_extract_np_cpp', PACKAGE = 'tall', sent_id, token_id, head_token_id, dep_rel, upos, terms, ngram_max, max_gap)
}

extract_svo_cpp <- function(sent_id, token_id, head_token_id, dep_rel, upos, terms) {
    .Call('_tall_extract_svo_cpp', PACKAGE = 'tall', sent_id, token_id, head_token_id, dep_rel, upos, terms)
}

calc_reciprocal_sum_cpp <- function(ngram_words, word_freq_names, word_freq_values) {
    .Call('_tall_calc_reciprocal_sum_cpp', PACKAGE = 'tall', ngram_words, word_freq_names, word_freq_values)
}

count_lexical_cpp <- function(ngram_pos, lexical_pos) {
    .Call('_tall_count_lexical_cpp', PACKAGE = 'tall', ngram_pos, lexical_pos)
}

generate_ngrams_cpp <- function(sentences_terms, sentences_pos, max_ngram, lexical_pos) {
    .Call('_tall_generate_ngrams_cpp', PACKAGE = 'tall', sentences_terms, sentences_pos, max_ngram, lexical_pos)
}

syntactic_complexity_cpp <- function(doc_id, sent_id, token_id, head_token_id, dep_rel, upos) {
    .Call('_tall_syntactic_complexity_cpp', PACKAGE = 'tall', doc_id, sent_id, token_id, head_token_id, dep_rel, upos)
}

txt_recode_cpp <- function(x, from, to, na_rm = FALSE) {
    .Call('_tall_txt_recode_cpp', PACKAGE = 'tall', x, from, to, na_rm)
}

txt_recode_ngram_cpp <- function(x, compound, ngram, sep = " ") {
    .Call('_tall_txt_recode_ngram_cpp', PACKAGE = 'tall', x, compound, ngram, sep)
}

txt_recode_batch_cpp <- function(x, from, to, na_rm = FALSE) {
    .Call('_tall_txt_recode_batch_cpp', PACKAGE = 'tall', x, from, to, na_rm)
}

