% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_np.R
\name{extract_noun_phrases}
\alias{extract_noun_phrases}
\title{Extract Noun Phrases via Dependency Parsing (C++ backend)}
\usage{
extract_noun_phrases(
  sent_id,
  token_id,
  head_token_id,
  dep_rel,
  upos,
  terms,
  ngram_max = 5L,
  max_gap = 3L
)
}
\arguments{
\item{sent_id}{Integer vector of sentence IDs}

\item{token_id}{Integer vector of token IDs}

\item{head_token_id}{Integer vector of head token IDs}

\item{dep_rel}{Character vector of dependency relations}

\item{upos}{Character vector of universal POS tags}

\item{terms}{Character vector of term values (lemma or token)}

\item{ngram_max}{Maximum phrase length (default 5)}

\item{max_gap}{Maximum gap between token positions (default 3)}
}
\value{
A data.frame with columns keyword and ngram
}
\description{
Extract Noun Phrases via Dependency Parsing (C++ backend)
}
